using Autofac;
using GZY.Quartz.MUI.Extensions;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using MS.Component.Aop;
using MS.Component.Jwt;
using MS.DbContexts;
using MS.Models.Automapper;
using MS.Services;
using MS.UnitOfWork;
using MS.WebApi.Filters;
using MS.WebApi.stockdbModels;
using MS.WebApi.waynedbModels;
using MS.WebCore;
using MS.WebCore.MultiLanguages;
using Npgsql.EntityFrameworkCore.PostgreSQL;

namespace MS.WebApi
{
    public class Startup
    {
        //public Startup(IConfiguration configuration)
        //{
        //    Configuration = configuration;
        //}
        public ILifetimeScope AutofacContainer { get; private set; }

        public Startup(IWebHostEnvironment env)
        {
            // In ASP.NET Core 3.0 `env` will be an IWebHostingEnvironment, not IHostingEnvironment.
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true)
                .AddEnvironmentVariables();
            Configuration = builder.Build();
        }
        //autofacDI
        public void ConfigureContainer(ContainerBuilder builder)
        {
            //עIBaseServiceIRoleServiceӿڼӦʵ
            //builder.RegisterType<BaseService>().As<IBaseService>().InstancePerLifetimeScope();
            //builder.RegisterType<RoleService>().As<IRoleService>().InstancePerLifetimeScope();

            //עaop 
            //ҵƴ˽ȥҵӿںʵעᣬҲҵ˴
            builder.AddAopService(ServiceExtensions.GetAssemblyName());
        }
        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            //ӶԱػ֧
            services.AddMultiLanguages();

            services.AddCors(options =>
            {
                //options.AddPolicy("AllowSpecificOrigin",
                //    builder => builder.WithOrigins("http://tkhs.net.cn:8091")//http://ipos.biz:8091
                //                      .AllowAnyHeader()
                //                      .AllowAnyMethod()
                //                      .AllowCredentials()

                //                      );
                //    //options.AddPolicy("AllowAllOrigins", 
                //    //    builder => builder.SetIsOriginAllowed(_ => true)
                //    //.AllowAnyHeader()
                //    //.AllowAnyMethod()
                //    //.AllowCredentials()
                //    //);
            });

            services.AddQuartzUI();
            services
                .AddControllers(options =>
                {
                    //  options.Filters.Add<ApiResultFilter>();
                    options.Filters.Add<ApiExceptionFilter>();
                })
                .AddDataAnnotationsLocalization(options =>
                {
                    options.DataAnnotationLocalizerProvider = (type, factory) =>
                    factory.Create(typeof(SharedResource));//עӱػԴṩLocalizerprovider
                });

            //ע
            //services.AddCorsPolicy(Configuration);
            //עwebcoreվҪã
            services.AddWebCoreService(Configuration);

            //עṤԪͬʱעDBContext
            services.AddUnitOfWorkService<MSDbContext>(options => { options.UseNpgsql(Configuration.GetSection("ConectionStrings:MSDbContext").Value); });
            services.AddUnitOfWorkService<AccountingContext>(options => { options.UseNpgsql(Configuration.GetSection("ConectionStrings:AccountingService").Value); });
            services.AddUnitOfWorkService<SQLiteContext>(options => { options.UseSqlite("Data Source=D:\\smartfueltool\\smartfueltool.db"); });
            services.AddUnitOfWorkService<AuthContext>(options => { options.UseNpgsql(Configuration.GetSection("ConectionStrings:AuthServiceDatabase").Value); });
            services.AddDbContext<stockdbContext>(options =>
     options.UseMySql(Configuration.GetSection("ConectionStrings:StockdbDatabase").Value));
            services.AddDbContext<waynedbContext>(options =>
  options.UseMySql(Configuration.GetSection("ConectionStrings:WaynedbDatabase").Value));
            //עautomapper
            services.AddAutomapperService();

            //עjwt
            services.AddJwtService(Configuration);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            app.UseQuartz(); //ʱ
            app.UseMultiLanguage(Configuration);//ӶԱػ֧

            app.UseRouting();

            app.UseCors(WebCoreExtensions.MyAllowSpecificOrigins);//ӿ

            app.UseAuthentication();//֤м

            app.UseAuthorization();
            app.UseCors("AllowSpecificOrigin");
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
